@@ -78,7 +78,7 @@ wm 32 0x021b001c 0x00008000
 wm 32 0x021b002c 0x000026d2
 wm 32 0x021b0030 0x003F1023
 
-SETUP_1GIB_2GIB_4GIB
+SETUP_MDASP_MDCTL
 
 wm 32 0x021b001c 0x04088032
 wm 32 0x021b001c 0x0408803a
