@@ -1,5 +1,6 @@
 #include <common.h>
 #include <command.h>
+#include <password.h>
 #include <environment.h>
 #include <shell.h>
 
@@ -266,6 +267,8 @@ int run_shell(void)
 	int len;
 	int rc = 1;
 
+	login();
+
 	for (;;) {
 		len = readline (CONFIG_PROMPT, console_buffer, CONFIG_CBSIZE);
 
