@@ -47,6 +47,7 @@
 #include <kfifo.h>
 #include <malloc.h>
 #include <matrix_keypad.h>
+#include <linux/err.h>
 
 /*
  * Keypad Controller registers (halfword)
@@ -390,7 +391,7 @@ static int __init imx_keypad_probe(struct device_d *dev)
 	struct imx_keypad *keypad;
 	const struct matrix_keymap_data *keymap_data = dev->platform_data;
 	struct console_device *cdev;
-	int error, i;
+	int i;
 
 	if (!keymap_data) {
 		pr_err("no keymap defined\n");
@@ -401,6 +402,8 @@ static int __init imx_keypad_probe(struct device_d *dev)
 
 	keypad->dev = dev;
 	keypad->mmio_base = dev_request_mem_region(dev, 0);
+	if (IS_ERR(keypad->mmio_base))
+		return PTR_ERR(keypad->mmio_base);
 
 	if(!keypad->fifo_size)
 		keypad->fifo_size = 50;
