@@ -445,15 +445,15 @@ static int nand_block_checkbad(struct mtd_info *mtd, loff_t ofs, int getchip,
 {
 	struct nand_chip *chip = mtd->priv;
 
-#ifdef CONFIG_NAND_BBT
-	if (!chip->bbt)
-		return chip->block_bad(mtd, ofs, getchip);
+	if (IS_ENABLED(CONFIG_NAND_BBT)) {
+		if (!chip->bbt)
+			return chip->block_bad(mtd, ofs, getchip);
+
+		/* Return info from the table */
+		return nand_isbad_bbt(mtd, ofs, allowbbt);
+	}
 
-	/* Return info from the table */
-	return nand_isbad_bbt(mtd, ofs, allowbbt);
-#else
 	return chip->block_bad(mtd, ofs, getchip);
-#endif
 }
 
 /* Wait for the ready pin, after a command. The timeout is caught later. */
