@@ -26,42 +26,15 @@
 #include <envfs.h>
 #include <linux/mtd/mtd.h>
 
-struct of_partition {
-	struct list_head list;
-	char *nodepath;
-	struct device_d *dev;
-	struct device_node *of_partitions;
-};
-
-static LIST_HEAD(of_partition_list);
-
 static int environment_probe(struct device_d *dev)
 {
 	char *path;
 	int ret;
 
-	ret = of_find_path(dev->device_node, "device-path", &path);
+	ret = of_find_path(dev->device_node, "device-path", &path, OF_FIND_PATH_FLAGS_BB);
 	if (ret)
 		return ret;
 
-	/*
-	 * The environment support is not bad block aware, hence we
-	 * have to use the .bb device. Test if we have a nand device
-	 * and if yes, append .bb to the filename.
-	 */
-	if (!strncmp(path, "/dev/", 5)) {
-		struct cdev *cdev;
-		char *cdevname;
-
-		cdevname = path + 5;
-		cdev = cdev_by_name(cdevname);
-		if (cdev && cdev->mtd && mtd_can_have_bb(cdev->mtd)) {
-			char *bbpath = asprintf("%s.bb", path);
-			free(path);
-			path = bbpath;
-		}
-	}
-
 	dev_info(dev, "setting default environment path to %s\n", path);
 
 	default_environment_path_set(path);
